//---------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	DssBuilder.h: --- A selectlib-compliant DataSubspace object
//
//	DESCRIPTION:
//		A class `DssBuilder' is defined instances of which can convert
//		selectlib-compliant selection expressions [1] into DataSubspace
//		objects. However, not all possible selection expressions can be
//		transformed. In other words, the list of possible DataSubspace
//		specifications forms a subset of the data selections supported by the
//		selectlib package. In case of a conversion error, the construction of
//		the DataSubspace object will fail with a thrown exception that has
//		to be caught in the calling context or with an ordinary errorcondition
//		(see comments in SelectError.h for details).
//
//		There is one global instance of a DssBuilder object called
//			dssBuilder
//		that applications such as `evselect' are supposed to use.
//		A typical example usage is:
//
//			#include "DssBuilder.h"	//	DSS builder functionalities
//			#include "dss.h"		//	DSS specifications
//			...
//			string selexpr = "PHA in [100:200],(300:400)"
//			DataSubspace dss = dssBuilder.dss(selexpr);
//
//			... do clever things with `dss'
//
//	CHANGE HISTORY:
//		UL, 1999-11-16: 1.0.0: - first version
//
//	REFERENCES:
//		[1]: Documentation of selectlib package, U. Lammers
//
//	AUTHOR:
//		U. Lammers (UL), Tue Nov 16 09:56:55 MET 1999
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/dsslib/src/build/DssBuilder.h,v 1.2 2009/05/18 11:32:48 aibarra Exp $
//

#ifndef DSS_BUILDER_H
#define DSS_BUILDER_H

#include <string>
#include <list>

#include "DataSubspace.h"
#include "Dal.h"
#include "SelectError.h"
#include "Region.h"
#include "Tracer.h"

using namespace SelectError;

typedef Range<double>	DRange;
typedef std::list<DRange>	DRangeList;

namespace DssParser {
	void lexinit(const std::string &expression, const Table *table);	//	initialize the lexer
	int	yyparse();  //	parse the expression
        extern int yydebug;
}

class DssBuilder {
public:
	//	exception classes
	class InvalidExpression : public SelectException {
    public:
        InvalidExpression(const std::string &s) : SelectException(s) {}
			//
			//	thrown if provided expression cannot be transformed
			//	into a DSS specification
			//
    };

	class ArithmeticException : public SelectException {
	public:
		ArithmeticException(const std::string &s) : SelectException(s) {}
			//
			//	thrown in case of arithmetic errors in the selection
			//	expression such as
			//		+ PHA in [1/0:2]
			//		+ BETA in [-2^-3.1:20]
			//
	};

	class MiscException : public SelectException {
	public:
		MiscException(const std::string &s) : SelectException(s) {}
			//
			//	thrown in case of misellaneous error conditions
			//
	};


	DssBuilder();
		//
		//	PURPOSE:
		//		Construct DssBuilder object
		//


	virtual DataSubspace dss();
		//
		//	PURPOSE:
		//		Return empty DataSubspace object
		//


	virtual DataSubspace dss(const std::string &expression, const Table *table = 0);
		//
		//	PURPOSE:
		//		Construct a DataSubspace object from a given selection
		//		expression. If the expression cannot be transformed
		//		the construction fails with an error message or
		//		an `InvalidExpression' exception is thrown.
		//
		//	ARGUMENTS:
		//		expression:	selectlib-compliant expression which drives the
		//					construction of the DataSubspace object
		//		table	  : pointer to a Dal table to which the filtering
		//					has been applied
		//


	virtual DataSubspace getDss(const std::string &expression, const Table *table = 0);
		//
		//	PURPOSE:
		//		Construct a DataSubspace object from a given selection
		//		expression. If the expression cannot be transformed
		//		the construction fails with an error message or
		//		an `InvalidExpression' exception is thrown.
		//
		//	ARGUMENTS:
		//		expression:	selectlib-compliant expression which drives the
		//					construction of the DataSubspace object
                //
                //              e: Exception return in case of error.
                //
		//		table	  : pointer to a Dal table to which the filtering
		//					has been applied
		//


	virtual ~DssBuilder();
		//
		//	PURPOSE:
		//		Delete object constructed with above constructor(s).
		//
private:
	//	data
        DataSubspace                    *_currentDss;
        std::vector<DataSubspace *>          _dssVector;
	DScomp				*_currentComp;

	//	region
	GeoRegion			*_region;
	Component			*_currentRegionComponent;
	FShape				*_currentRegion;
	std::string				_xAxisName, _yAxisName;

	friend int DssParser::yyparse();
private:
	//	methods
	DScomp *currentComponent();
	GeoRegion *newRegion();
	DScomp *newComponent();
        DataSubspace *newDss();
        DataSubspace *andDss();
        DataSubspace *orDss();
        DataSubspace *negateDss();
        unsigned axisCount;
        void clear();

	Component	*currentRegionComponent();
	FShape		*currentRegion();
	void		regionAxes(const std::string &a1, const std::string &a2);
	std::string		xAxisName() const { PROC RETURN _xAxisName; }
	std::string		yAxisName() const { PROC RETURN _yAxisName; }
	void		addRegion(FShape *region);
        std::string          Axis2dName(const DSS::DSfilterComp &fcomp);

private:
        void initDss();
	void initRegion();
};

#define dssBuilder	dssBuilderFunc()	//	single global instance of
						//	dssBuilder object
DssBuilder &dssBuilderFunc();

#endif
